@@ -51,6 +51,7 @@
  *
  *  Different generations of CPUs and SIMD capabilities can be enabled or disabled with the following macros:
  *
+ *  - `SZ_USE_WESTMERE=?` - whether to use SSE4.2 & AES-NI instructions on x86_64.
  *  - `SZ_USE_HASWELL=?` - whether to use AVX2 instructions on x86_64.
  *  - `SZ_USE_SKYLAKE=?` - whether to use AVX-512 instructions on x86_64.
  *  - `SZ_USE_ICE=?` - whether to use AVX-512 VBMI & wider AES instructions on x86_64.
@@ -65,8 +66,8 @@
 #define STRINGZILLA_H_
 
 #define STRINGZILLA_H_VERSION_MAJOR 4
-#define STRINGZILLA_H_VERSION_MINOR 0
-#define STRINGZILLA_H_VERSION_PATCH 15
+#define STRINGZILLA_H_VERSION_MINOR 2
+#define STRINGZILLA_H_VERSION_PATCH 0
 
 #include "types.h"        // `sz_size_t`, `sz_bool_t`, `sz_ordering_t`
 #include "compare.h"      // `sz_equal`, `sz_order`
@@ -114,12 +115,15 @@ SZ_INTERNAL sz_size_t sz_capabilities_to_strings_implementation_(sz_capability_t
         {sz_cap_serial_k, "serial"},
         {sz_cap_parallel_k, "parallel"},
         //
+        {sz_cap_westmere_k, "westmere"},
+        {sz_cap_goldmont_k, "goldmont"},
         {sz_cap_haswell_k, "haswell"},
         {sz_cap_skylake_k, "skylake"},
         {sz_cap_ice_k, "ice"},
         //
         {sz_cap_neon_k, "neon"},
         {sz_cap_neon_aes_k, "neon+aes"},
+        {sz_cap_neon_sha_k, "neon+sha"},
         {sz_cap_sve_k, "sve"},
         {sz_cap_sve2_k, "sve2"},
         {sz_cap_sve2_aes_k, "sve2+aes"},
@@ -156,6 +160,8 @@ SZ_INTERNAL sz_capability_t sz_capability_from_string_implementation_(char const
     if (sz_equal_null_terminated_serial(name, "serial") == sz_true_k) return sz_cap_serial_k;
     if (sz_equal_null_terminated_serial(name, "parallel") == sz_true_k) return sz_cap_parallel_k;
     // x86
+    if (sz_equal_null_terminated_serial(name, "westmere") == sz_true_k) return sz_cap_westmere_k;
+    if (sz_equal_null_terminated_serial(name, "goldmont") == sz_true_k) return sz_cap_goldmont_k;
     if (sz_equal_null_terminated_serial(name, "haswell") == sz_true_k) return sz_cap_haswell_k;
     if (sz_equal_null_terminated_serial(name, "skylake") == sz_true_k) return sz_cap_skylake_k;
     if (sz_equal_null_terminated_serial(name, "ice") == sz_true_k) return sz_cap_ice_k;
@@ -166,6 +172,9 @@ SZ_INTERNAL sz_capability_t sz_capability_from_string_implementation_(char const
     if (sz_equal_null_terminated_serial(name, "neon_aes") == sz_true_k ||
         sz_equal_null_terminated_serial(name, "neon+aes") == sz_true_k)
         return sz_cap_neon_aes_k;
+    if (sz_equal_null_terminated_serial(name, "neon_sha") == sz_true_k ||
+        sz_equal_null_terminated_serial(name, "neon+sha") == sz_true_k)
+        return sz_cap_neon_sha_k;
     if (sz_equal_null_terminated_serial(name, "sve2_aes") == sz_true_k ||
         sz_equal_null_terminated_serial(name, "sve2+aes") == sz_true_k)
         return sz_cap_sve2_aes_k;
@@ -210,6 +219,25 @@ SZ_INTERNAL sz_cptr_t sz_capabilities_to_string_implementation_(sz_capability_t
     return buf;
 }
 
+SZ_PUBLIC sz_capability_t sz_capabilities_comptime_implementation_(void) {
+    return (sz_capability_t)(                   //
+        (sz_cap_neon_k * SZ_USE_NEON) |         //
+        (sz_cap_neon_aes_k * SZ_USE_NEON_AES) | //
+        (sz_cap_neon_sha_k * SZ_USE_NEON_SHA) | //
+        (sz_cap_sve_k * SZ_USE_SVE) |           //
+        (sz_cap_sve2_k * SZ_USE_SVE2) |         //
+        (sz_cap_sve2_aes_k * SZ_USE_SVE2_AES) | //
+        (sz_cap_westmere_k * SZ_USE_WESTMERE) | //
+        (sz_cap_goldmont_k * SZ_USE_GOLDMONT) | //
+        (sz_cap_haswell_k * SZ_USE_HASWELL) |   //
+        (sz_cap_skylake_k * SZ_USE_SKYLAKE) |   //
+        (sz_cap_ice_k * SZ_USE_ICE) |           //
+        (sz_cap_cuda_k * SZ_USE_CUDA) |         //
+        (sz_cap_kepler_k * SZ_USE_KEPLER) |     //
+        (sz_cap_hopper_k * SZ_USE_HOPPER) |     //
+        (sz_cap_serial_k));
+}
+
 #if SZ_IS_64BIT_ARM_
 
 /*  Compiling the next section one may get: selected processor does not support system register name 'id_aa64zfr0_el1'.
@@ -235,26 +263,30 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_arm_(void) {
     // On Apple Silicon, `mrs` is not allowed in user-space, so we need to use the `sysctl` API.
     uint32_t supports_neon = 0;
     uint32_t supports_neon_aes = 0;
+    uint32_t supports_neon_sha = 0;
     size_t size = sizeof(supports_neon);
     if (sysctlbyname("hw.optional.neon", &supports_neon, &size, NULL, 0) != 0) supports_neon = 0;
     if (sysctlbyname("hw.optional.arm.FEAT_AES", &supports_neon_aes, &size, NULL, 0) != 0) supports_neon_aes = 0;
+    if (sysctlbyname("hw.optional.arm.FEAT_SHA256", &supports_neon_sha, &size, NULL, 0) != 0) supports_neon_sha = 0;
 
     return (sz_capability_t)(                       //
         (sz_cap_neon_k * (supports_neon)) |         //
         (sz_cap_neon_aes_k * (supports_neon_aes)) | //
+        (sz_cap_neon_sha_k * (supports_neon_sha)) | //
         (sz_cap_serial_k));
 
 #elif defined(SZ_IS_LINUX_)
 
     // Read CPUID registers directly
     unsigned long id_aa64isar0_el1 = 0, id_aa64isar1_el1 = 0, id_aa64pfr0_el1 = 0, id_aa64zfr0_el1 = 0;
-    unsigned supports_neon = 0, supports_neon_aes = 0, supports_sve = 0, supports_sve2 = 0, supports_sve2_aes = 0;
+    unsigned supports_neon = 0, supports_neon_aes = 0, supports_neon_sha = 0, supports_sve = 0, supports_sve2 = 0,
+             supports_sve2_aes = 0;
     sz_unused_(id_aa64isar0_el1);
     sz_unused_(id_aa64isar1_el1);
     sz_unused_(id_aa64pfr0_el1);
     sz_unused_(id_aa64zfr0_el1);
 
-#if SZ_USE_NEON || SZ_USE_SVE || SZ_USE_SVE2 || SZ_USE_NEON_AES || SZ_USE_SVE2_AES
+#if SZ_USE_NEON || SZ_USE_SVE || SZ_USE_SVE2 || SZ_USE_NEON_AES || SZ_USE_NEON_SHA || SZ_USE_SVE2_AES
     // Now let's unpack the status flags from ID_AA64ISAR0_EL1
     // https://developer.arm.com/documentation/ddi0601/2024-03/AArch64-Registers/ID-AA64ISAR0-EL1--AArch64-Instruction-Set-Attribute-Register-0?lang=en
     __asm__ __volatile__("mrs %0, ID_AA64ISAR0_EL1" : "=r"(id_aa64isar0_el1));
@@ -273,7 +305,10 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_arm_(void) {
     // That's a really weird way to encode lack of NEON support, but it's important to
     // check in case we are running on R-profile CPUs.
     supports_neon = ((id_aa64pfr0_el1 >> 20) & 0xF) != 0xF;
+    // AES, bits [7:4] of ID_AA64ISAR0_EL1
     supports_neon_aes = ((id_aa64isar0_el1 >> 4) & 0xF) >= 1;
+    // SHA2, bits [15:12] of ID_AA64ISAR0_EL1
+    supports_neon_sha = ((id_aa64isar0_el1 >> 12) & 0xF) >= 1;
 
 #if SZ_USE_SVE || SZ_USE_SVE2 || SZ_USE_SVE2_AES
     // SVE, bits [35:32] of ID_AA64PFR0_EL1
@@ -293,6 +328,7 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_arm_(void) {
     return (sz_capability_t)(                       //
         (sz_cap_neon_k * (supports_neon)) |         //
         (sz_cap_neon_aes_k * (supports_neon_aes)) | //
+        (sz_cap_neon_sha_k * (supports_neon_sha)) | //
         (sz_cap_sve_k * (supports_sve)) |           //
         (sz_cap_sve2_k * (supports_sve2)) |         //
         (sz_cap_sve2_aes_k * (supports_sve2_aes)) | //
@@ -315,7 +351,7 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_arm_(void) {
 
 SZ_PUBLIC sz_capability_t sz_capabilities_implementation_x86_(void) {
 
-#if SZ_USE_HASWELL || SZ_USE_SKYLAKE || SZ_USE_ICE
+#if SZ_USE_WESTMERE || SZ_USE_GOLDMONT || SZ_USE_HASWELL || SZ_USE_SKYLAKE || SZ_USE_ICE
 
     /// The states of 4 registers populated for a specific "cpuid" assembly call
     union four_registers_t {
@@ -367,10 +403,17 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_x86_(void) {
     unsigned supports_avx512vbmi2 = os_avx512_enabled && ((info7.named.ecx & 0x00000040u) != 0);
     unsigned supports_vaes = os_avx512_enabled && ((info7.named.ecx & 0x00000200u) != 0);
 
-    return (sz_capability_t)(                                                                                //
-        (sz_cap_haswell_k * supports_avx2) |                                                                 //
-        (sz_cap_skylake_k * (supports_avx512f && supports_avx512vl && supports_avx512bw && supports_vaes)) | //
-        (sz_cap_ice_k * (supports_avx512vbmi && supports_avx512vbmi2)) |                                     //
+    // Check for SSE4.2, AES-NI, and SHA-NI (Function ID 1 and 7)
+    unsigned supports_sse42 = ((info1.named.ecx & 0x00100000u) != 0);
+    unsigned supports_aesni = ((info1.named.ecx & 0x02000000u) != 0);
+    unsigned supports_shani = ((info7.named.ebx & 0x20000000u) != 0); // SHA-NI bit 29 in EBX from CPUID(7,0)
+
+    return (sz_capability_t)(                                                               //
+        (sz_cap_westmere_k * (supports_sse42 && supports_aesni)) |                          //
+        (sz_cap_goldmont_k * (supports_shani)) |                                            //
+        (sz_cap_haswell_k * (supports_avx2)) |                                              //
+        (sz_cap_skylake_k * (supports_avx512f && supports_avx512vl && supports_avx512bw)) | //
+        (sz_cap_ice_k * (supports_avx512vbmi && supports_avx512vbmi2 && supports_vaes)) |   //
         (sz_cap_serial_k));
 #else
     return sz_cap_serial_k;
@@ -379,11 +422,11 @@ SZ_PUBLIC sz_capability_t sz_capabilities_implementation_x86_(void) {
 #endif // SZ_IS_64BIT_X86_
 
 /**
- *  @brief Function to determine the SIMD capabilities of the current 64-bit x86 machine at @b runtime.
+ *  @brief Function to determine the SIMD capabilities of the current CPU at @b runtime.
  *  @return A bitmask of the SIMD capabilities represented as a `sz_capability_t` enum value.
  *  @note Excludes parallel-processing & GPGPU capabilities, which are detected separately in StringZillas.
  */
-SZ_PUBLIC sz_capability_t sz_capabilities_implementation_(void) {
+SZ_PUBLIC sz_capability_t sz_capabilities_runtime_implementation_(void) {
 #if SZ_IS_64BIT_X86_
     return sz_capabilities_implementation_x86_();
 #elif SZ_IS_64BIT_ARM_
@@ -399,6 +442,8 @@ SZ_DYNAMIC int sz_dynamic_dispatch(void);
 SZ_DYNAMIC int sz_version_major(void);
 SZ_DYNAMIC int sz_version_minor(void);
 SZ_DYNAMIC int sz_version_patch(void);
+SZ_DYNAMIC sz_capability_t sz_capabilities_comptime(void);
+SZ_DYNAMIC sz_capability_t sz_capabilities_runtime(void);
 SZ_DYNAMIC sz_capability_t sz_capabilities(void);
 SZ_DYNAMIC sz_cptr_t sz_capabilities_to_string(sz_capability_t caps);
 SZ_DYNAMIC void sz_dispatch_table_update(sz_capability_t caps);
@@ -409,7 +454,11 @@ SZ_DYNAMIC int sz_dynamic_dispatch(void) { return 0; }
 SZ_PUBLIC int sz_version_major(void) { return STRINGZILLA_H_VERSION_MAJOR; }
 SZ_PUBLIC int sz_version_minor(void) { return STRINGZILLA_H_VERSION_MINOR; }
 SZ_PUBLIC int sz_version_patch(void) { return STRINGZILLA_H_VERSION_PATCH; }
-SZ_PUBLIC sz_capability_t sz_capabilities(void) { return sz_capabilities_implementation_(); }
+SZ_PUBLIC sz_capability_t sz_capabilities_comptime(void) { return sz_capabilities_comptime_implementation_(); }
+SZ_PUBLIC sz_capability_t sz_capabilities_runtime(void) { return sz_capabilities_runtime_implementation_(); }
+SZ_PUBLIC sz_capability_t sz_capabilities(void) {
+    return (sz_capability_t)(sz_capabilities_comptime_implementation_() & sz_capabilities_runtime_implementation_());
+}
 SZ_PUBLIC sz_cptr_t sz_capabilities_to_string(sz_capability_t caps) {
     return sz_capabilities_to_string_implementation_(caps);
 }
